<?php

/**
 * Shopgate GmbH
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file AFL_license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to interfaces@shopgate.com so we can send you a copy immediately.
 *
 * @author    Shopgate GmbH, Schloßstraße 10, 35510 Butzbach <interfaces@shopgate.com>
 * @copyright Shopgate GmbH
 * @license   http://opensource.org/licenses/AFL-3.0 Academic Free License ("AFL"), in the version 3.0
 */
class ShopgateDb
{
    const ENGINE_PDO = 'DbPDO';
    const ENGINE_MYSQLI = 'DbMySQLi';

    /** @var Db | DbCore | DbPDOCore | MySQLCore | DbMySQLiCore $db */
    protected $db;
    /** @var PDO | MySQLi | Resource  */
    protected $link;

    /**
     * @return Db | DbCore | DbPDOCore | MySQLCore
     */
    public function getInstance()
    {
        if (is_null($this->db)) {
            /** @noinspection PhpUndefinedClassInspection */
            $this->db = Db::getInstance();
        }

        return $this->db;
    }

    /**
     * @param Db | DbCore | DbPDOCore | MySQLCore $dbInstance
     *
     * @return $this
     */
    public function setInstance(Db $dbInstance)
    {
        $this->db = $dbInstance;

        return $this;
    }

    /**
     * Starts db transaction, base on used engine
     */
    public function beginTransaction()
    {
        if ($this->isPDOUsed()) {
            $this->getLink()->beginTransaction();
        } elseif ($this->isMysqliUsed()) {
            $this->getLink()->autocommit(false);
        } else {
            ShopgateLogger::getInstance()->log(
                'Transactions not supported by DB engine.', ShopgateLogger::LOGTYPE_DEBUG
            );
        }
    }

    /**
     * Commit transaction if possible
     */
    public function commit()
    {
        if ($this->engineSupportsTransactions()) {
            $this->getLink()->commit();

            if ($this->isMysqliUsed()) {
                $this->getLink()->autocommit(true);
            }
        }
    }

    /**
     * Rollback transaction if possible
     */
    public function rollback()
    {
        if ($this->engineSupportsTransactions()) {
            $this->getLink()->rollback();

            if ($this->isMysqliUsed()) {
                $this->getLink()->autocommit(true);
            }
        }
    }

    /**
     * @return bool
     */
    public function engineSupportsTransactions()
    {
        return $this->isPDOUsed() || $this->isMysqliUsed();
    }

    /**
     * @return bool
     */
    public function isMysqliUsed()
    {
        return get_class($this->getInstance()) == self::ENGINE_MYSQLI;
    }

    /**
     * @return bool
     */
    public function isPDOUsed()
    {
        return get_class($this->getInstance()) == self::ENGINE_PDO;
    }

    /**
     * @return PDO | MySQLi | Resource
     */
    private function getLink()
    {
        if (is_null($this->link)) {
            $this->link = $this->db->connect();
        }

        return $this->link;
    }

    /**
     * Executes SQL query based on selected type
     *
     * @param string $table
     * @param array $data
     * @param string $type (INSERT, INSERT IGNORE, REPLACE, UPDATE).
     * @param string $where
     * @param int $limit
     * @param bool $use_cache
     * @param bool $use_null
     * @return bool
     * @throws PrestaShopDatabaseException
     */
    public function autoExecute($table, $data, $type, $where = '', $limit = 0, $use_cache = true, $use_null = false)
    {
        if (version_compare(_PS_VERSION_, '1.7', '>=')) {
            $type = strtoupper($type);
            switch ($type) {
                case 'INSERT' :
                    return $this->getInstance()->insert($table, $data, $use_null, $use_cache, Db::INSERT, false);

                case 'INSERT IGNORE' :
                    return $this->getInstance()->insert($table, $data, $use_null, $use_cache, Db::INSERT_IGNORE, false);

                case 'REPLACE' :
                    return $this->getInstance()->insert($table, $data, $use_null, $use_cache, Db::REPLACE, false);

                case 'UPDATE' :
                    return $this->getInstance()->update($table, $data, $where, $limit, $use_null, $use_cache, false);

                default :
                    throw new PrestaShopDatabaseException('Wrong argument (miss type) in Db::autoExecute()');
            }
        } else {
            $this->getInstance()->autoExecute($table, $data, $type, $where, $limit = 0, $use_cache, $use_null);
        }
    }

    /**
     * Filter SQL query within a blacklist
     *
     * @param string $table Table where insert/update data
     * @param array $values Data to insert/update
     * @param string $type INSERT or UPDATE
     * @param string $where WHERE clause, only for UPDATE (optional)
     * @param int $limit LIMIT clause (optional)
     * @return bool
     * @throws PrestaShopDatabaseException
     */
    public function autoExecuteWithNullValues($table, $values, $type, $where = '', $limit = 0)
    {
        return $this->autoExecute($table, $values, $type, $where, $limit, 0, true);
    }
}
