<?php
/**
 * Shopgate GmbH
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file AFL_license.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to interfaces@shopgate.com so we can send you a copy immediately.
 *
 * @author    Shopgate GmbH, Schloßstraße 10, 35510 Butzbach <interfaces@shopgate.com>
 * @copyright Shopgate GmbH
 * @license   http://opensource.org/licenses/AFL-3.0 Academic Free License ("AFL"), in the version 3.0
 */

/**
 * Class ShopgateItemsBasePrice
 */
class ShopgateItemsBasePrice
{
    /** @var ShopgatePrestashopVersion */
    private $prestashopVersion;

    /** @var string */
    private $currencyIsoCode;

    /**
     * @param ShopgatePrestashopVersion $prestashopVersion
     * @param string                    $currencyIsoCode
     */
    public function __construct(ShopgatePrestashopVersion $prestashopVersion, $currencyIsoCode)
    {
        $this->prestashopVersion = $prestashopVersion;
        $this->currencyIsoCode   = $currencyIsoCode;
    }

    /**
     * @param float  $basePrice
     * @param string $basePriceUnit
     *
     * @return string
     */
    public function getBasePriceFormatted($basePrice, $basePriceUnit)
    {
        $basePrice = number_format($basePrice, 2);

        return $basePrice . ' ' . $this->currencyIsoCode . (!empty($basePriceUnit) ? ' / ' . $basePriceUnit : '');
    }

    /**
     * @param float      $basePrice
     * @param bool       $useTax
     * @param float|null $taxRate
     *
     * @return float
     */
    public function calculateBasePrice($basePrice, $useTax, $taxRate = null)
    {
        if ($useTax && $taxRate !== null) {
            $basePrice = $this->addTaxes($basePrice, $taxRate);
        }

        return $this->round($basePrice);
    }

    /**
     * @param float      $childProductPriceExclTax
     * @param float      $parentProductUnitPriceRatio
     * @param int        $unitPriceImpact (1 => Increase, 0 => None, -1 => Reduction)
     * @param bool       $useTax
     * @param float|null $taxRate
     *
     * @return float
     */
    public function calculateChildBasePrice(
        $childProductPriceExclTax,
        $parentProductUnitPriceRatio,
        $unitPriceImpact,
        $useTax,
        $taxRate = null
    ) {
        $basePrice = ($childProductPriceExclTax / $parentProductUnitPriceRatio);

        if ($useTax && $taxRate !== null) {
            $basePrice = $this->addTaxes($basePrice, $taxRate);

            if (version_compare($this->prestashopVersion->getVersion(), '1.6.0.11', '>=')) {
                $unitPriceImpact = $this->addTaxes($unitPriceImpact, $taxRate);
            }
        }

        $basePrice += $unitPriceImpact;

        return $this->round($basePrice);
    }

    /**
     * @param int $unitPriceImpact
     *
     * @return bool
     */
    public function basePriceAffectsChildren($unitPriceImpact)
    {
        return isset($unitPriceImpact) && $unitPriceImpact != 0;
    }

    /**
     * @param Product $product
     *
     * @return bool
     */
    public function basePriceActive($product)
    {
        return $product->unit_price != 0;
    }

    /**
     * @param float $price
     *
     * @return float
     */
    private function round($price)
    {
        return (float)round($price * 100) / 100;
    }

    /**
     * @param float $price
     * @param float $taxRate
     *
     * @return float
     */
    private function addTaxes($price, $taxRate)
    {
        return $price + ($price * $taxRate / 100);
    }
}